/*
    SPDX-FileCopyrightText: 2011 Aleix Pol Gonzalez <aleixpol@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "ilanguagecheckprovider.h"

KDevelop::ILanguageCheckProvider::~ILanguageCheckProvider()
{}
