/*
    SPDX-FileCopyrightText: 2006-2007 Andreas Pakulat <apaku@gmx.de>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "ioutputview.h"
#include "debug.h"

namespace KDevelop
{

IOutputView::~IOutputView()
{}

}

