/**
 * "type" : {"toString" : "DOMWindow" }
 */
var w = window;

/**
 * "type" : {"toString" : "Document" }
 */
var doc = document;

/**
 * "type" : {"toString" : "Element" }
 */
var el = document.createElement("p");

/**
 * "type": {"toString" : "string" }
 */
var name = el.getAttribute("id");