/*
    SPDX-FileCopyrightText: 2017 Aleix Pol <aleixpol@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "iruntime.h"

using namespace KDevelop;

KDevelop::IRuntime::~IRuntime() = default;

#include "moc_iruntime.cpp"
