/*
    SPDX-FileCopyrightText: 2007 Dukju Ahn <dukjuahn@gmail.com>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "ioutputviewmodel.h"

namespace KDevelop
{

IOutputViewModel::~IOutputViewModel()
{}

}

