/*
    SPDX-FileCopyrightText: 2015 Kevin Funk <kfunk@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "itoolviewactionlistener.h"

using namespace KDevelop;

IToolViewActionListener::~IToolViewActionListener()
{
}
