/*
    SPDX-FileCopyrightText: 2019 Daniel Mensinger <daniel@mensinger-ka.de>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "mesonactionbase.h"

MesonRewriterActionBase::MesonRewriterActionBase() {}
MesonRewriterActionBase::~MesonRewriterActionBase() {}

void MesonRewriterActionBase::parseResult(QJsonObject) {}
