{% extends "cpp_qobject_pimpl_header.h" %}

{% block qobject_default_include %}
#include <QWidget>
{% endblock qobject_default_include %}

{% block qobject_baseclass %}QWidget{% endblock qobject_baseclass %}
