/*
    SPDX-FileCopyrightText: 2007 Alexander Dymo <adymo@kdevelop.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "ilanguagecontroller.h"

namespace KDevelop {

ILanguageController::ILanguageController(QObject *parent)
    :QObject(parent)
{
}

}

#include "moc_ilanguagecontroller.cpp"
